import unittest

import electrum_ecc as ecc
from electrum_ecc import ECPubkey, ECPrivkey
from electrum_ecc import _libsecp256k1
from electrum_ecc.util import sha256


bfh = bytes.fromhex


class TestEcdsa(unittest.TestCase):

    def test_verify_enforces_low_s(self):
        # privkey = ecc.ECPrivkey(bytes.fromhex("d473e2ec218dca8e3508798f01cdfde0135fc79d95526b12e3537fe57e479ac1"))
        # r, low_s = privkey.ecdsa_sign(msg32, sigencode=lambda x, y: (x,y))
        # pubkey = ecc.ECPubkey(privkey.get_public_key_bytes())
        pubkey = ecc.ECPubkey(bytes.fromhex("03befe4f7c92eaed73fb8eddac28c6191c87c6a3546bf8dc09643e1e10bc6f5ab0"))
        msg32 = sha256(b"hello there")
        r = 29658118546717807188148256874354333643324863178937517286987684851194094232509
        # low-S
        low_s = 9695211969150896589566136599751503273246834163278279637071703776634378000266
        sig64_low_s = (
            int.to_bytes(r, length=32, byteorder="big") +
            int.to_bytes(low_s, length=32, byteorder="big"))
        self.assertTrue(pubkey.ecdsa_verify(sig64_low_s, msg32))
        # high-S
        high_s = ecc.CURVE_ORDER - low_s
        sig64_high_s = (
            int.to_bytes(r, length=32, byteorder="big") +
            int.to_bytes(high_s, length=32, byteorder="big"))
        self.assertFalse(pubkey.ecdsa_verify(sig64_high_s, msg32))
        self.assertTrue(pubkey.ecdsa_verify(sig64_high_s, msg32, enforce_low_s=False))

    def test_ecdsa_sign(self):
        eckey1 = ecc.ECPrivkey(bfh('7e1255fddb52db1729fc3ceb21a46f95b8d9fe94cc83425e936a6c5223bb679d'))
        sig1 = eckey1.ecdsa_sign(bfh('5a548b12369a53faaa7e51b5081829474ebdd9c924b3a8230b69aa0be254cd94'),
                                 sigencode=ecc.ecdsa_der_sig_from_r_and_s)
        self.assertEqual('3044022066e7d6a954006cce78a223f5edece8aaedcf3607142e9677acef1cfcb91cfdde022065cb0b5401bf16959ce7b785ea7fd408be5e4cb7d8f1b1a32c78eac6f73678d9', sig1.hex())

        eckey2 = ecc.ECPrivkey(bfh('c7ce8c1462c311eec24dff9e2532ac6241e50ae57e7d1833af21942136972f23'))
        sig2 = eckey2.ecdsa_sign(bfh('642a2e66332f507c92bda910158dfe46fc10afbf72218764899d3af99a043fac'),
                                 sigencode=ecc.ecdsa_der_sig_from_r_and_s)
        self.assertEqual('30440220618513f4cfc87dde798ce5febae7634c23e7b9254a1eabf486be820f6a7c2c4702204fef459393a2b931f949e63ced06888f35e286e446dc46feb24b5b5f81c6ed52', sig2.hex())

    def test_ecdsa_sign_without_r_value_grinding(self):
        eckey1 = ecc.ECPrivkey(bfh('7e1255fddb52db1729fc3ceb21a46f95b8d9fe94cc83425e936a6c5223bb679d'))
        sig1 = eckey1.ecdsa_sign(
            bfh('5a548b12369a53faaa7e51b5081829474ebdd9c924b3a8230b69aa0be254cd94'),
            sigencode=ecc.ecdsa_der_sig_from_r_and_s, grind_r_value=False,
        )
        self.assertEqual('3045022100902a288b98392254cd23c0e9a49ac6d7920f171b8249a48e484b998f1874a2010220723d844826828f092cf400cb210c4fa0b8cd1b9d1a7f21590e78e022ff6476b9', sig1.hex())

    def test_ecdsa_verify_der_sig(self):
        eckey1 = ecc.ECPrivkey.from_secret_scalar(10877953613094898331777953037035807584681004899710037363338258839373864549145)
        msg32 = bytes(32)
        der_sig = bfh('3045022100cfd454a1215fdea463201a7a32c146c1cec54b60b12d47e118a2add41366cec602203e7875d23cc80f958e45298bb8369d4422acfbc1c317353eebe02c89206b3e73')
        # der sig must be converted for ecdsa_verify:
        sig64 = ecc.ecdsa_sig64_from_der_sig(der_sig)
        self.assertTrue(eckey1.ecdsa_verify(sig64, msg32))
        # so this won't work:
        self.assertFalse(eckey1.ecdsa_verify(der_sig, msg32))

        fake_msg = sha256(b"satoshi")
        self.assertFalse(eckey1.ecdsa_verify(sig64, fake_msg))



import sys

from setuptools.build_meta import *  # noqa: F401, F403
from setuptools.build_meta import build_wheel

backend_class = build_wheel.__self__.__class__


class _CustomBuildMetaBackend(backend_class):
    def run_setup(self, setup_script="setup.py"):
        flags = []
        if self.config_settings and "electrum_ecc.dont_compile" in self.config_settings:
            flags.append(
                f"--electrum_ecc.dont_compile={self.config_settings['electrum_ecc.dont_compile']}"
            )
        if flags:
            sys.argv = sys.argv[:1] + ["build_ext"] + flags + sys.argv[1:]
        return super().run_setup(setup_script)

    def build_wheel(
        self, wheel_directory, config_settings=None, metadata_directory=None
    ):
        self.config_settings = config_settings
        return super().build_wheel(wheel_directory, config_settings, metadata_directory)


build_wheel = _CustomBuildMetaBackend().build_wheel



import csv
from ctypes import (
    c_int, c_char_p, c_size_t, c_void_p, create_string_buffer,
)
import io
import unittest

import electrum_ecc as ecc
from electrum_ecc import ECPubkey, ECPrivkey
from electrum_ecc import _libsecp256k1
from electrum_ecc.util import sha256, bip340_tagged_hash


class TestSchnorr(unittest.TestCase):

    def test_vectors_from_bip0340(self):
        bip0340_vectors = """index,secret key,public key,aux_rand,message,signature,verification result,comment
0,0000000000000000000000000000000000000000000000000000000000000003,F9308A019258C31049344F85F89D5229B531C845836F99B08601F113BCE036F9,0000000000000000000000000000000000000000000000000000000000000000,0000000000000000000000000000000000000000000000000000000000000000,E907831F80848D1069A5371B402410364BDF1C5F8307B0084C55F1CE2DCA821525F66A4A85EA8B71E482A74F382D2CE5EBEEE8FDB2172F477DF4900D310536C0,TRUE,
1,B7E151628AED2A6ABF7158809CF4F3C762E7160F38B4DA56A784D9045190CFEF,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,0000000000000000000000000000000000000000000000000000000000000001,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,6896BD60EEAE296DB48A229FF71DFE071BDE413E6D43F917DC8DCF8C78DE33418906D11AC976ABCCB20B091292BFF4EA897EFCB639EA871CFA95F6DE339E4B0A,TRUE,
2,C90FDAA22168C234C4C6628B80DC1CD129024E088A67CC74020BBEA63B14E5C9,DD308AFEC5777E13121FA72B9CC1B7CC0139715309B086C960E18FD969774EB8,C87AA53824B4D7AE2EB035A2B5BBBCCC080E76CDC6D1692C4B0B62D798E6D906,7E2D58D8B3BCDF1ABADEC7829054F90DDA9805AAB56C77333024B9D0A508B75C,5831AAEED7B44BB74E5EAB94BA9D4294C49BCF2A60728D8B4C200F50DD313C1BAB745879A5AD954A72C45A91C3A51D3C7ADEA98D82F8481E0E1E03674A6F3FB7,TRUE,
3,0B432B2677937381AEF05BB02A66ECD012773062CF3FA2549E44F58ED2401710,25D1DFF95105F5253C4022F628A996AD3A0D95FBF21D468A1B33F8C160D8F517,FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF,FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF,7EB0509757E246F19449885651611CB965ECC1A187DD51B64FDA1EDC9637D5EC97582B9CB13DB3933705B32BA982AF5AF25FD78881EBB32771FC5922EFC66EA3,TRUE,test fails if msg is reduced modulo p or n
4,,D69C3509BB99E412E68B0FE8544E72837DFA30746D8BE2AA65975F29D22DC7B9,,4DF3C3F68FCC83B27E9D42C90431A72499F17875C81A599B566C9889B9696703,00000000000000000000003B78CE563F89A0ED9414F5AA28AD0D96D6795F9C6376AFB1548AF603B3EB45C9F8207DEE1060CB71C04E80F593060B07D28308D7F4,TRUE,
5,,EEFDEA4CDB677750A420FEE807EACF21EB9898AE79B9768766E4FAA04A2D4A34,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,6CFF5C3BA86C69EA4B7376F31A9BCB4F74C1976089B2D9963DA2E5543E17776969E89B4C5564D00349106B8497785DD7D1D713A8AE82B32FA79D5F7FC407D39B,FALSE,public key not on the curve
6,,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,FFF97BD5755EEEA420453A14355235D382F6472F8568A18B2F057A14602975563CC27944640AC607CD107AE10923D9EF7A73C643E166BE5EBEAFA34B1AC553E2,FALSE,has_even_y(R) is false
7,,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,1FA62E331EDBC21C394792D2AB1100A7B432B013DF3F6FF4F99FCB33E0E1515F28890B3EDB6E7189B630448B515CE4F8622A954CFE545735AAEA5134FCCDB2BD,FALSE,negated message
8,,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,6CFF5C3BA86C69EA4B7376F31A9BCB4F74C1976089B2D9963DA2E5543E177769961764B3AA9B2FFCB6EF947B6887A226E8D7C93E00C5ED0C1834FF0D0C2E6DA6,FALSE,negated s value
9,,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,0000000000000000000000000000000000000000000000000000000000000000123DDA8328AF9C23A94C1FEECFD123BA4FB73476F0D594DCB65C6425BD186051,FALSE,sG - eP is infinite. Test fails in single verification if has_even_y(inf) is defined as true and x(inf) as 0
10,,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,00000000000000000000000000000000000000000000000000000000000000017615FBAF5AE28864013C099742DEADB4DBA87F11AC6754F93780D5A1837CF197,FALSE,sG - eP is infinite. Test fails in single verification if has_even_y(inf) is defined as true and x(inf) as 1
11,,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,4A298DACAE57395A15D0795DDBFD1DCB564DA82B0F269BC70A74F8220429BA1D69E89B4C5564D00349106B8497785DD7D1D713A8AE82B32FA79D5F7FC407D39B,FALSE,sig[0:32] is not an X coordinate on the curve
12,,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFC2F69E89B4C5564D00349106B8497785DD7D1D713A8AE82B32FA79D5F7FC407D39B,FALSE,sig[0:32] is equal to field size
13,,DFF1D77F2A671C5F36183726DB2341BE58FEAE1DA2DECED843240F7B502BA659,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,6CFF5C3BA86C69EA4B7376F31A9BCB4F74C1976089B2D9963DA2E5543E177769FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEBAAEDCE6AF48A03BBFD25E8CD0364141,FALSE,sig[32:64] is equal to curve order
14,,FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFC30,,243F6A8885A308D313198A2E03707344A4093822299F31D0082EFA98EC4E6C89,6CFF5C3BA86C69EA4B7376F31A9BCB4F74C1976089B2D9963DA2E5543E17776969E89B4C5564D00349106B8497785DD7D1D713A8AE82B32FA79D5F7FC407D39B,FALSE,public key is not a valid X coordinate because it exceeds the field size
"""
        with io.StringIO(bip0340_vectors) as f:
            csvreader = csv.reader(f)
            next(csvreader)  # skip first line
            for idx, seckey, pubkey, aux_rand, message, signature, expected_res, comment in csvreader:
                idx = int(idx)
                with self.subTest(msg=f"{idx=}"):
                    try:
                        pubkey = ECPubkey(bytes.fromhex("02" + pubkey))
                    except ecc.InvalidECPointException:
                        if idx in (5, 14, ):  # expected for these tests
                            continue
                        raise
                    msg32 = bytes.fromhex(message)
                    signature = bytes.fromhex(signature)
                    if seckey:
                        seckey = ECPrivkey(bytes.fromhex(seckey))
                        aux_rand = bytes.fromhex(aux_rand)
                        sig_created = seckey.schnorr_sign(msg32, aux_rand32=aux_rand)
                        self.assertEqual(signature, sig_created)
                    is_sig_good = pubkey.schnorr_verify(signature, msg32)
                    expected_res = True if expected_res == "TRUE" else False
                    self.assertEqual(expected_res, is_sig_good)

    def test_sign_schnorr_aux_rand(self):
        seckey = ECPrivkey(bytes.fromhex("B7E151628AED2A6ABF7158809CF4F3C762E7160F38B4DA56A784D9045190CFEF"))
        msg32 = sha256(b"hello there")
        sig1 = seckey.schnorr_sign(msg32, aux_rand32=None)
        sig2 = seckey.schnorr_sign(msg32, aux_rand32=b"\x00" * 32)
        self.assertEqual(sig1, sig2)
        sig3 = seckey.schnorr_sign(msg32, aux_rand32=bytes(range(32)))
        self.assertNotEqual(sig1, sig3)

    def test_y_parity_malleability(self):
        # BIP-0340 says:
        # > A hypothetical verification algorithm that treats points as public keys,
        # > and takes the point P directly as input would fail any time a point with
        # > odd Y is used. While it is possible to correct for this by negating points
        # > with odd Y coordinate before further processing, this would result in a scheme
        # > where every (message, signature) pair is valid for two public keys (a type of
        # > malleability that exists for ECDSA as well, but we don't wish to retain).
        # > We avoid these problems by treating just the X coordinate as public key.
        #
        # As the API in ecc.py treats points as public keys, this malleability exists here:
        seckey = ECPrivkey.from_secret_scalar(1337)
        pubkey1 = ECPubkey(seckey.get_public_key_bytes())
        pubkey2 = ecc.CURVE_ORDER * ecc.GENERATOR + (-1) * pubkey1
        self.assertNotEqual(pubkey1.get_public_key_bytes(True), pubkey2.get_public_key_bytes(True))
        self.assertEqual(pubkey1.get_public_key_bytes(True)[1:], pubkey2.get_public_key_bytes(True)[1:])
        msg32 = sha256(b"hello there")
        sig = seckey.schnorr_sign(msg32, aux_rand32=None)
        self.assertTrue(pubkey1.schnorr_verify(sig, msg32))
        self.assertTrue(pubkey2.schnorr_verify(sig, msg32))

    def test_bip340_tagged_hash(self):
        try:
            _libsecp256k1.secp256k1_tagged_sha256.argtypes = [c_void_p, c_char_p, c_char_p, c_size_t, c_char_p, c_size_t]
            _libsecp256k1.secp256k1_tagged_sha256.restype = c_int
        except (OSError, AttributeError):
            raise Exception('libsecp256k1 library too old: missing secp256k1_tagged_sha256 method')

        def bip340_tagged_hash__from_libsecp(tag: bytes, msg: bytes) -> bytes:
            assert isinstance(tag, bytes), type(tag)
            assert isinstance(msg, bytes), type(msg)
            thash = create_string_buffer(32)
            ret = _libsecp256k1.secp256k1_tagged_sha256(
                _libsecp256k1.ctx, thash, tag, len(tag), msg, len(msg))
            assert 1 == ret, ret
            thash = bytes(thash)
            return thash

        data = (
            (b"", b""),
            (b"", b"hello there"),
            (b"mytag", b""),
            (b"mytag", b"hello there"),
            (bytes(range(256)) * 10, bytes(range(256)) * 50),
            (bytes(range(256)) * 1000, bytes(range(256)) * 5000),
        )
        for tag, msg in data:
            self.assertEqual(bip340_tagged_hash__from_libsecp(tag, msg),
                             bip340_tagged_hash(tag, msg))




# based on https://github.com/ludbb/secp256k1-py/blob/f5e455227bf1e833128adf80de8ee0ebcebf218c/setup.py

import errno
import logging
import os
import os.path
import platform
import shutil
import subprocess
import sys

import setuptools
from setuptools import setup

try:
    # needs new setuptools: >=70.1.0
    from setuptools.command.bdist_wheel import bdist_wheel, safer_name, get_platform
except ImportError as e1:
    try:
        # or needs old wheel: <0.44
        from wheel.bdist_wheel import bdist_wheel, safer_name, get_platform
    except ImportError as e2:
        try:
            import wheel
            wheel_version = wheel.__version__
        except Exception:
            wheel_version = None
        raise ImportError(
            "Cannot import bdist_wheel from either setuptools or wheel. "
            "This is likely due to having old 'setuptools' (<70.1) with new 'wheel' (>=0.44) package. "
            "This combination is not supported. "
            f"Found: {setuptools.__version__=}, {wheel_version=}."
        ) from e2


_logger = logging.getLogger("electrum_ecc")
MAKE = 'gmake' if platform.system() in ['FreeBSD', 'OpenBSD'] else 'make'

ELECTRUM_ECC_DONT_COMPILE = None
for arg in sys.argv[:]:
    if arg.startswith("--electrum_ecc.dont_compile="):
        value = arg.split("=", 1)[1]
        sys.argv.remove(arg)
        ELECTRUM_ECC_DONT_COMPILE = value
        break
if ELECTRUM_ECC_DONT_COMPILE is None:
    ELECTRUM_ECC_DONT_COMPILE = os.getenv("ELECTRUM_ECC_DONT_COMPILE") or ""
_logger.info(f"Checking env var: {ELECTRUM_ECC_DONT_COMPILE=!r}")
if ELECTRUM_ECC_DONT_COMPILE == "":  # unset
    IS_COMPILING_LIB = sys.platform != "win32"
elif ELECTRUM_ECC_DONT_COMPILE == "1":  # user explicitly opted out
    IS_COMPILING_LIB = False
else:  # any other value
    IS_COMPILING_LIB = True


def absolute(*paths):
    op = os.path
    return op.realpath(op.abspath(op.join(op.dirname(__file__), *paths)))


def compile_secp(build_dir: str) -> None:
    _logger.info(f"Going to compile libsecp ourselves. {build_dir=}")

    if not os.path.exists(absolute('libsecp256k1')):
        raise Exception("missing git submodule secp256k1")

    deps_msg = (
        "For compiling libsecp256k1, besides a C compiler, "
        "you might be missing one of: autoconf automake libtool."
        "\n"
        "To opt out of compiling libsecp, set ELECTRUM_ECC_DONT_COMPILE=1 (as an environment variable). "
        "If you opt out, you will still need to make sure libsecp256k1 is available "
        "(either system-installed or by copying a .so/.dll/.dylib into electrum_ecc/ )"
    )

    if not os.path.exists(absolute('libsecp256k1/configure')):
        # configure script hasn't been generated yet
        autogen = absolute('libsecp256k1/autogen.sh')
        os.chmod(absolute(autogen), 0o755)
        try:
            subprocess.check_call([autogen], cwd=absolute('libsecp256k1'))
        except (subprocess.CalledProcessError, OSError) as e:
            raise Exception(f"autogen.sh failed. {deps_msg}") from e

    for filename in [
        'libsecp256k1/configure',
        'libsecp256k1/build-aux/compile',
        'libsecp256k1/build-aux/config.guess',
        'libsecp256k1/build-aux/config.sub',
        'libsecp256k1/build-aux/depcomp',
        'libsecp256k1/build-aux/install-sh',
        'libsecp256k1/build-aux/missing',
        'libsecp256k1/build-aux/test-driver',
    ]:
        try:
            os.chmod(absolute(filename), 0o755)
        except OSError as e:
            # some of these files might not exist depending on autoconf version
            if e.errno != errno.ENOENT:
                # If the error isn't 'No such file or directory' something
                # else is wrong and we want to know about it
                raise

    cmd = [
        absolute('libsecp256k1/configure'),
        '--enable-shared',
        '--disable-static',
        '--disable-dependency-tracking',
        '--with-pic',
        '--prefix',
        os.path.abspath(build_dir),
        '--enable-module-recovery',
        '--enable-module-extrakeys',
        '--enable-module-schnorrsig',
        '--enable-experimental',
        '--enable-module-ecdh',
        '--enable-benchmark=no',
        '--enable-tests=no',
        '--enable-openssl-tests=no',
        '--enable-exhaustive-tests=no',
    ]

    _logger.info('Running configure: {}'.format(' '.join(cmd)))
    try:
        subprocess.check_call(cmd, cwd=build_dir)

        subprocess.check_call([MAKE], cwd=build_dir)
        subprocess.check_call([MAKE, 'install'], cwd=build_dir)
    except (subprocess.CalledProcessError, OSError) as e:
        raise Exception(f"error compiling libsecp. {deps_msg}") from e


class Custom_bdist_wheel(bdist_wheel):

    def finalize_options(self):
        if IS_COMPILING_LIB:
            # Inject the platform name, e.g. "linux_x86_64".
            # This will result in the final build artifact being named e.g.
            #   electrum_ecc-0.0.2-py3-none-linux_x86_64.whl
            self.plat_name = get_platform(self.bdist_dir)
            # note: we don't set the python "impl tag" or the "abi tag", as the C lib we build
            #       does not depend on them (it is not a "C extension" as we don't static link cpython).
        bdist_wheel.finalize_options(self)

    def _build_and_copy_secp_lib(self):
        _build_cmd = self.get_finalized_command('build')
        build_dir = os.path.join(_build_cmd.build_base, "temp_libsecp")
        target_dir = os.path.join(self.bdist_dir, safer_name(self.distribution.get_name()))
        for dir_ in (build_dir, target_dir):
            try:
                os.makedirs(dir_)
            except OSError as e:
                if e.errno != errno.EEXIST:
                    raise

        compile_secp(build_dir)

        build_temp_libs = os.path.join(build_dir, "lib")
        for fname in os.listdir(build_temp_libs):
            # note: we skip the versioned .so files (e.g. "*.so.2.2.0"), as wheels don't
            #       support symlinks and we would end up with ~3 copies of the same file
            if fname.endswith(".so") or fname.endswith(".dll") or fname.endswith(".dylib"):
                srcpath = os.path.join(build_temp_libs, fname)
                _logger.info(f"copying file {srcpath!r} to {target_dir=}")
                shutil.copy2(srcpath, target_dir)

    def run(self):
        if IS_COMPILING_LIB:
            self._build_and_copy_secp_lib()
        bdist_wheel.run(self)


setup(
    cmdclass={
        'bdist_wheel': Custom_bdist_wheel,
    },
)



import unittest

import electrum_ecc as ecc
from electrum_ecc.util import sha256


bfh = bytes.fromhex

class TestBasics(unittest.TestCase):

    def test_ecc_sanity(self):
        G = ecc.GENERATOR
        n = G.order()
        self.assertEqual(ecc.CURVE_ORDER, n)
        inf = n * G
        self.assertEqual(ecc.POINT_AT_INFINITY, inf)
        self.assertTrue(inf.is_at_infinity())
        self.assertFalse(G.is_at_infinity())
        self.assertEqual(11 * G, 7 * G + 4 * G)
        self.assertEqual((n + 2) * G, 2 * G)
        self.assertEqual((n - 2) * G, -2 * G)
        A = (n - 2) * G
        B = (n - 1) * G
        C = n * G
        D = (n + 1) * G
        self.assertFalse(A.is_at_infinity())
        self.assertFalse(B.is_at_infinity())
        self.assertTrue(C.is_at_infinity())
        self.assertTrue((C * 5).is_at_infinity())
        self.assertFalse(D.is_at_infinity())
        self.assertEqual(inf, C)
        self.assertEqual(inf, A + 2 * G)
        self.assertEqual(inf, D + (-1) * G)
        self.assertNotEqual(A, B)
        self.assertEqual(2 * G, inf + 2 * G)
        self.assertEqual(inf, 3 * G + (-3 * G))

    def test_ecdh(self):
        def ecdh(privkey: ecc.ECPrivkey, pubkey: ecc.ECPubkey) -> bytes:
            pt = (privkey.secret_scalar * pubkey)
            return sha256(pt.get_public_key_bytes(compressed=True))

        # Alice's perspective:
        a = ecc.ECPrivkey.from_secret_scalar(11)
        B = ecc.ECPubkey(bfh("022b4ea0a797a443d293ef5cff444f4979f06acfebd7e86d277475656138385b6c"))
        dh_secret_1 = ecdh(a, B)
        # Bob's perspective:
        A = ecc.ECPubkey(bfh("03774ae7f858a9411e5ef4246b70c65aac5649980be5c17891bbec17895da008cb"))
        b = ecc.ECPrivkey.from_secret_scalar(19)
        dh_secret_2 = ecdh(b, A)

        self.assertEqual(dh_secret_1, dh_secret_2)
        self.assertEqual("739a9fccc559f41588003e0d44812950ddb9ce23d2d2077e426be4dea5d8d2b4", dh_secret_1.hex())
